#include "testApp.h"

// looks like devices 4 and 5 are the two usb cameras.

//128.208.7.237

//--------------------------------------------------------------
void testApp::setup(){
	camWidth = 640;
	camHeight = 360;
	margin = 20;
	
#ifdef _USE_LIVE_VIDEO
	vidGrabber.setVerbose(true);
	vidGrabber.initGrabber(camWidth,camHeight);
#else
	vidPlayer.loadMovie("lamp.mov");
	vidPlayer.play();
#endif
	
    colorImg.allocate(camWidth,camHeight);
	origColorImg.allocate(camWidth, camHeight);
	grayImage.allocate(camWidth,camHeight);
	grayBg.allocate(camWidth,camHeight);
	grayDiff.allocate(camWidth,camHeight);
	
	bLearnBakground = true;
	threshold = 12;
	
	oldXMin = oldXMax = oldX = oldY = -1;
	
	sender.setup(HOST,PORT);
	
	
	// color tracking
	one.pos = ofxVec2f(0,0);
	
    colorImgHSV.allocate(camWidth,camHeight);   //our HSB image that will house the color image and deal out the Hue, Saturation and brightness
	
    hueImg.allocate(camWidth,camHeight);        //Hue map
    satImg.allocate(camWidth,camHeight);        //saturation map
    briImg.allocate(camWidth,camHeight);        //brightness map, not gonna be used but necessary 
	
    reds.allocate(camWidth, camHeight);         //our postRange image basically
	
    colorTrackedPixelsRed = new unsigned char [camWidth * camHeight];     //rangeImage
	
    trackedTextureRed.allocate(camWidth, camHeight, GL_LUMINANCE);        //final output
	
	showDebug = true;
	numNotes = 19;
	numInstruments = 5;
	drawKeyboard = false;
	colorImgTex.allocate(camWidth, camHeight, GL_RGB);
}

//--------------------------------------------------------------
void testApp::update(){
	bool bNewFrame = false;
	
#ifdef _USE_LIVE_VIDEO
	vidGrabber.grabFrame();
	bNewFrame = vidGrabber.isFrameNew();
#else
	vidPlayer.idleMovie();
	bNewFrame = vidPlayer.isFrameNew();
#endif
	
	if (bNewFrame){
		
#ifdef _USE_LIVE_VIDEO
		colorImg.setFromPixels(vidGrabber.getPixels(), camWidth,camHeight);
#else
		colorImg.setFromPixels(vidPlayer.getPixels(), camWidth,camHeight);
#endif
		colorImgTex.loadData(colorImg.getPixels(), camWidth, camHeight, GL_RGB);
		
		// color tracking
		
		colorImgHSV = colorImg;                                                 //now we stuff the colorImg into our HSB image
		colorImgHSV.convertRgbToHsv();                                          //now we convert the colorImg inside colorImgHSV into HSV
		
		colorImgHSV.convertToGrayscalePlanarImages(hueImg, satImg, briImg);     //distribute the hue, saturation and brightness to hueImg, satImg, and briImg
		
		
		hueImg.flagImageChanged(); 
		satImg.flagImageChanged();
		briImg.flagImageChanged();
		
		// ok cool, here we go:
		unsigned char * huePixels = hueImg.getPixels();                         //huePixels is now a raw array of pixels
		unsigned char * satPixels = satImg.getPixels();                         //satPixels is now a raw array of pixels  just like huePixels
		//unsigned char * briPixels = briImg.getPixels();
		int nPixels = camWidth * camHeight;                                     //get the number of pixels in the images since these raw images are continuous, so no breaks
		//so pixel number camWidth + 1 would be the first pixels in the second row of pixels of the image 
		
		
		
		for (int i = 0; i < nPixels; i++){                                           //let's go through every pixel in hue pixels
			if ((huePixels[i] >= one.hue - threshold && huePixels[i] <= one.hue + threshold) &&    //if the hue is of a certain range
				//(satPixels[i] >= one.sat - 24 && satPixels[i] <= one.sat + 200)){    //if the saturation is of a certain range
				(satPixels[i] >= one.sat - 10 && satPixels[i] <= one.sat + 220)){
				colorTrackedPixelsRed[i] = 255;                                      //mark this corresponding pixel white
			} else {
				colorTrackedPixelsRed[i] = 0;                                        //if it doesn't fit then color the corresponding pixel black
			}
		}
		
		reds.setFromPixels(colorTrackedPixelsRed, camWidth, camHeight);              //set reds from the colorTrackedPixelsRed array so it's all clean and openCv operable
		finderRed.findContours(reds, 50,nPixels/3, 1, false, true);                  //lets find one (1) blob in the grayscale openCv image reds
		
		trackedTextureRed.loadData(colorTrackedPixelsRed, camWidth, camHeight, GL_LUMINANCE);  //load up the data from the colorTrackedPixelsRed into a texture
		
		//------------------------------
		
		if(finderRed.blobs.size() > 0) {
			one.pos = ofxVec2f(finderRed.blobs[0].centroid.x,finderRed.blobs[0].centroid.y);  //if the blob exists, set it's associated color (one) to it's position
		}
	}
}

//--------------------------------------------------------------
void testApp::draw(){

	
	
    if(finderRed.nBlobs > 0) {   
        char tempStr1[255];
        sprintf(tempStr1, "x : %f\ny : %f", finderRed.blobs[0].centroid.x, finderRed.blobs[0].centroid.y);  
        ofDrawBitmapString(tempStr1, 20, 250); //draw the string
		interestBlob = finderRed.blobs[0];
        xx = 1.0 * interestBlob.centroid.x / camWidth;
		yy = 1.0 * interestBlob.centroid.y / camHeight;
		
	   if(oldX < 0 || xx > oldXMax || xx < oldXMin || oldY - yy > 0.005 || oldY - yy < - 0.005) {
		   oldY = yy;
		   oldX = xx;
		   oldXMax = ceil(xx * numNotes);
		   oldXMin = floor(xx * numNotes);
		   if(oldXMax == oldXMin) {
			   oldXMax++;
		   }
		   oldXMax /= numNotes;
		   oldXMin	/= numNotes;
		   
		   cout << "bright spot! " << xx << ", " << yy << "\n";
		   
		   ofxOscMessage m;
		   m.setAddress(PATH);
		   
		   m.addFloatArg(xx);
		   m.addFloatArg(yy);
		   
		   sender.sendMessage(m);
	   }
	} else {
		oldX = oldXMin = oldXMax = xx = yy = -1;	
	}
	
	if(showDebug) {
		ofBackground(100,100,100);
		ofSetColor(0xffffff);
		colorImg.draw(margin, margin);
		colorImgHSV.draw(margin * 2 + camWidth, margin);
		trackedTextureRed.draw(margin, margin * 2 + camHeight);
		finderRed.draw(margin, margin * 2 + camHeight);
		
		// draw a grid over the hand
		float gridCellWidth = camWidth * 1.0 / numNotes;
		for(int i = 1; i < numNotes; i++) {
			ofRect(gridCellWidth * i + margin, margin * 2 + camHeight, 1, camHeight);
		}
		
		float gridCellHeight = camHeight * 1.0 / numInstruments;
		for(int i = 1; i < numInstruments; i++) {
			ofRect(0 + margin, gridCellHeight * i + margin * 2 + camHeight, camWidth, 1);
		}
	} else {
		// draw a grid over the hand
		ofBackground(0, 0, 0);
		
		int winWidth = ofGetWidth() - margin * 2;
		int winHeight = ofGetHeight() - margin * 2;
		// draw the window
		ofSetColor(0xffffff);
		colorImgTex.draw(margin, margin, winWidth, winHeight);
		
		ofEnableAlphaBlending();
		// highlight that square
		float gridCellWidth = winWidth * 1.0 / numNotes;
		float gridCellHeight = winHeight * 1.0 / numInstruments;
		float tmp;
		
		if(xx > 0) {
			ofFill();
			int hilightedColumn = xx * numNotes;
			int hilightedRow = yy * numInstruments;
			ofSetColor(221, 0, 164, 127);
			ofRect(hilightedColumn * gridCellWidth + margin + 1, hilightedRow * gridCellHeight + margin + 1, 
				   gridCellWidth-1, gridCellHeight-1);
		}
		
		ofSetColor(255, 255, 255, 127);
		ofNoFill();
		
		for(int i = 1; i < numNotes; i++) {
			tmp = gridCellWidth * i;
			ofRect(tmp + margin, margin, 1, winHeight);
		}
		
		for(int i = 1; i < numInstruments; i++) {
			tmp = gridCellHeight * i;
			ofRect(margin, tmp + margin, winWidth, 1);
		}

		ofRect(margin, margin, winWidth, winHeight);
		ofDisableAlphaBlending();
	}
	
	// or, instead we can draw each blob individually,
	// this is how to get access to them:
	/*
    for (int i = 0; i < contourFinder.nBlobs; i++){
        //contourFinder.blobs[i].draw(camWidth + margin, margin * 3 + camHeight * 2);
		//contourFinder.blobs[i].draw(margin, margin);
		ofxCvBlob b = contourFinder.blobs[i];
    }*/
	
//	if(contourFinder.nBlobs > 0) {
//		ofFill();
//		ofSetColor(0x24eb24);
//		interestBlob = contourFinder.blobs[0];
//		//ofCircle(margin * 2 + camWidth + interestBlob.centroid.x, margin * 2 + camHeight + interestBlob.centroid.y, 5);
//		//ofCircle(margin + interestBlob.centroid.x, margin + interestBlob.centroid.y, 5);
//		ofCircle(margin + interestBlob.centroid.x, margin * 2 + camHeight + interestBlob.centroid.y, 5);
//		// TODO: send an OSC message here
//		
//		float xx = 1.0 * interestBlob.centroid.x / camWidth;
//		float yy = 1.0 * interestBlob.centroid.y / camHeight;
//		
//		if(oldX < 0 || xx > oldXMax || xx < oldXMin || oldY - yy > 0.005 || oldY - yy < - 0.005) {
//			oldY = yy;
//			oldX = xx;
//			oldXMax = ceil(xx * numNotes);
//			oldXMin = floor(xx * numNotes);
//			if(oldXMax == oldXMin) {
//				oldXMax++;
//			}
//			oldXMax /= numNotes;
//			oldXMin	/= numNotes;
//		
//			cout << "bright spot! " << xx << ", " << yy << "\n";
//
//			ofxOscMessage m;
//			m.setAddress(PATH);
//			
//			m.addFloatArg(xx);
//			m.addFloatArg(yy);
//			
//			sender.sendMessage(m);
//		}
//	} else {
//		oldX = oldXMin = oldXMax = -1;
//	}
	
	// finally, a report:
	
	/*
	ofSetColor(0xffffff);
	char reportStr[1024];
	sprintf(reportStr, "bg subtraction and blob detection\npress ' ' to capture bg\nthreshold %i (press: +/-)\nnum blobs found %i, fps: %f", threshold, contourFinder.nBlobs, ofGetFrameRate());
	ofDrawBitmapString(reportStr, 20, 600);
	*/
	
	
}


//--------------------------------------------------------------
void testApp::keyPressed  (int key){
	
	/* 
	 d for switch to and from debug (show each webcam grab)
	 
	 1 for increase num columns
	 2 for decrease num columns
	 
	 3 for increase num rows
	 4 for decrease num rows
	 
	 + for increase threshold
	 - for decrease threshold
	
	*/
	switch (key){
		case ' ':
			bLearnBakground = true;
			break;
		case '+':
			threshold ++;
			if (threshold > 255) threshold = 255;
			break;
		case '-':
			threshold --;
			if (threshold < 0) threshold = 0;
			break;
		case 'd':
			showDebug = !showDebug;
			break;
		case '1':
			numNotes++;
			break;		
		case '2':
			numNotes = max(1, numNotes - 1);
			break;
		case '3':
			numInstruments++;
			break;
		case '4':
			numInstruments = max(1, numInstruments - 1);
			break;		
		case 'k':
			drawKeyboard = !drawKeyboard;
			break;		
		case 'f':
			ofToggleFullscreen();
			break;					
	}
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y ){
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button){
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button){
	
    unsigned char * huePixels = hueImg.getPixels();  //teh hue
    unsigned char * satPixels = satImg.getPixels();  //teh saturation
	
    x = MIN(x,hueImg.width-1);     //find the smallest value out of those two so we don't crash if we click outside of the camera image
    y = MIN(y,hueImg.height-1);
	
    if(button == 0) {
        one.hue = huePixels[x+(y*hueImg.width)];  //set the hue
        one.sat = satPixels[x+(y*satImg.width)];  //set the sat
        /*one.bri = briPixels[x+(y*briImg.width)];*/
    }
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	
}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h){
	
}

