//
// SampleSpace.ck 
//
// The entry point for the SampleSpace program
//
// Author: Zack Brown <zacharyb@cs.washington.edu>,
// Author: Andrew Smith <andrewrs@cs.washington.edu>
// Modified: 2011-03-04
//

// What We're Implementing
// =======================
// DONE:  Make voices and/or voice selection modulate based on position data
// TODO:  Add some random variance (random ADSR envelopes?) for a more 'human' feel
//		  (or will pressure differences in the real glove be enough?)
// TODO:  Implement looping.  Have another piece of code monitor the NoteEvents
// and record them, then re-send notes and artifical NoteEvents in a loop
// To measure time, you can say "now => time t" to get a timestamp and subtract
// timestamps to yield type "dur"s



// Constants
48888 => int PORT_NUMBER;

// Create glove emulator
GloveEmulator g;
g.setPort(PORT_NUMBER);

// Initialize voice(s) and VoiceManager
VoiceManager vm;
(3, 3) => vm.setGridSize;
vm.setVoice(0, 0, new SineVoice);
vm.setVoice(0, 1, new JazzPianoVoice);
vm.setVoice(0, 2, new WowSynthVoice);
vm.setVoice(1, 0, new WeirdFMVoice);
(0.0, 0.0) => vm.setCurrentPosition;

// Set up limiter (to avoid clipping)
Dyno d;
d.limit();
// 0.0 => d.slopeAbove;
// .4 => d.thresh;

// Set up reverb (for effect)
NRev r;
.03 => r.mix;

// Add looping and recording
LoopManager loop;
(4, 2) => loop.setMeasure;     // (beats per measure, measures)
120    => loop.setBpm;         // beats per minute

// Create OSC Listener and show it the voice manager and loop manager to talk to
OSCManager osc;
vm   @=> osc.vm;
loop @=> osc.loop;

// Throw everything together
vm => r => d => loop.chuck => dac;


while (true) {
    10::second => now;
}