
//
// VoiceManager.ck
//
// VoiceManager accepts a list of Voices, then subsequently
// a series of "play" commands.  It handles the y value of
// the current position to change the active track
//
// Author(s): Zack Brown <zacharyb@cs.washington.edu>,
//            Andrew Smith <andrewrs@cs.washington.edu>
// Modified: 15 Mar 2011
//

public class VoiceManager extends UGen {
    
    // Keep a list of the voices for this, row-major order
    Voice @ voices[9];
    
    3 => int rows;
    3 => int cols;
    
    // Which voice is currently being played
    Voice @ currentVoice;
    
    
    
    // Set the size (and therefore the number of voices) for this setup
    public void setGridSize(int rows, int cols) {
        if (rows > 0 && cols > 0) {
            <<< "Setting grid size", "" >>>;
            
            rows => this.rows;
            cols => this.cols;
        
            new Voice[rows * cols] @=> voices;
            voices[0] @=> currentVoice;
        }
    }
    
    // Assigns a voice to a specific square.
    public void setVoice(int row, int col, Voice v) {
        if (row >= 0 && row < rows && col >= 0 && col < cols) {
            <<< "Assigning a voice", "" >>>;
            
            // Valid voice to set
            v @=> voices[row * cols + col];
            v => this;
            
            spork ~ v.play();
        }
    }
    
    // Given an (x,y) coordinate, find the voice that associated with that
    // square.
    private Voice getVoice(float x, float y) {
        // Find the row and column first
        (y * rows) $ int => int row;
        (x * cols) $ int => int col;

        return voices[row * cols + col];
    }
    
    // Translates from the current glove position to the voice that is currently
    // in control.
    public void setCurrentPosition(float x, float y) {
        (x, y) => getVoice @=> Voice v;
        
        if (v != currentVoice) {
            <<< "Setting current position/voice", "" >>>;
            
            // Since we're changing the voice, we should tell the outgoing
            // voice about the note off event.
            currentVoice.noteEvent @=> NoteEvent e;
            0 => e.on;
            e.signal();
            
            // Finally, change the voice
            v @=> currentVoice;
        }
    }
}